/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package jdbctest;

import java.io.File;
import java.io.FileReader;
import java.sql.Connection;
import java.sql.Date;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.LinkedList;
import java.util.List;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

/**name;
private Date dateOfDeath;
private int age;
 *
 * @author xkremser
 */
public class DeadBodyManagerImpl implements DeadBodyManager {

    public static final String INSERT_BODY = "INSERT INTO DEAD_BODY (name, dateOfDeath, age) VALUES (? ,? , ?)";
    public static final String FIND_BODIES_BY_NAME = "SELECT * FROM DEAD_BODY WHERE name LIKE ?";
    public static final String UPDATE_BODY = "UPDATE DEAD_BODY SET name = ?, dateOfDeath = ?, age = ? WHERE id = ?";
    public static final String DELETE_BODY = "DELETE FROM DEAD_BODY WHERE id = ?";

    private Connection initConnection() {
        Connection con = null;
        try {
            Properties prop = new Properties();
            prop.load(new FileReader("database"));
            String url = prop.getProperty("jdbcUrl");
            String login = prop.getProperty("login");
            String pass = prop.getProperty("password");
            return DriverManager.getConnection(url, login, pass);
        } catch (SQLException ex) {
            Logger.getLogger(DeadBodyManagerImpl.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            return con;
        }
    }

    private void closeConnection(Connection con) {
        try {
            con.close();
        } catch (SQLException ex) {
            Logger.getLogger(DeadBodyManagerImpl.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void createDeadBody(DeadBody body) {
        if (body == null) {
            throw new NullPointerException("body");
        }
        Connection con = initConnection();
        try {
            if (con == null) {
                System.out.println("con is null");
            }
            PreparedStatement st = con.prepareStatement(INSERT_BODY, Statement.RETURN_GENERATED_KEYS);
            st.setString(1, body.getName());
            st.setDate(2, new Date(body.getDateOfDeath().getTime()));
            st.setInt(3, body.getAge());
            st.execute();
            ResultSet rs = st.getGeneratedKeys();
            while (rs.next()) {
                body.setId(rs.getLong(1));
            }
        } catch (SQLException ex) {
            Logger.getLogger(DeadBodyManagerImpl.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            if (con != null) {
                closeConnection(con);
            }
        }

    }

    public boolean deleteDeadBody(DeadBody body) {
        if (body == null) {
            throw new NullPointerException("body");
        }
        Connection con = null;
        boolean success = true;
        try {
            con = DriverManager.getConnection("jdbc:derby://localhost:1527/sample", "app", "app");
            PreparedStatement st = con.prepareStatement(DELETE_BODY);
            st.setLong(1, body.getId());
            int modifiedRows = st.executeUpdate();
            if (modifiedRows != 1) {
                success = false;
            }
        } catch (SQLException ex) {
            Logger.getLogger(DeadBodyManagerImpl.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            if (con != null) {
                try {
                    con.close();
                } catch (SQLException ex) {
                    Logger.getLogger(DeadBodyManagerImpl.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
            return success;
        }
    }

    public boolean updateDeadBody(DeadBody body) {
        if (body == null) {
            throw new NullPointerException("body");
        }
        Connection con = null;
        boolean success = true;
        try {
            con = DriverManager.getConnection("jdbc:derby://localhost:1527/sample", "app", "app");
            PreparedStatement st = con.prepareStatement(UPDATE_BODY, Statement.RETURN_GENERATED_KEYS);
            st.setString(1, body.getName());
            st.setDate(2, new Date(body.getDateOfDeath().getTime()));
            st.setInt(3, body.getAge());
            st.setLong(4, body.getId());
            int modifiedRows = st.executeUpdate();
            if (modifiedRows != 1) {
                success = false;
            }

            ResultSet rs = st.getGeneratedKeys();
            while (rs.next()) {
                body.setId(rs.getLong(1));
            }

        } catch (SQLException ex) {
            Logger.getLogger(DeadBodyManagerImpl.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            if (con != null) {
                try {
                    con.close();
                } catch (SQLException ex) {
                    Logger.getLogger(DeadBodyManagerImpl.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
            return success;
        }
    }

    public List<DeadBody> findDeadBodyByName(String name) {
        if (name == null) {
            throw new NullPointerException("name");
        }
        Connection con = null;
        List<DeadBody> returnList = new LinkedList<DeadBody>();
        try {
            con = DriverManager.getConnection("jdbc:derby://localhost:1527/sample", "app", "app");
            PreparedStatement st = con.prepareStatement(FIND_BODIES_BY_NAME);
            st.setString(1, "%" + name + "%");
            ResultSet rs = st.executeQuery();

            while (rs.next()) {
                returnList.add(new DeadBody(rs.getLong(1), rs.getString(2), rs.getDate("DATEOFDEATH"), rs.getInt(4)));
            }
        } catch (SQLException ex) {
            Logger.getLogger(DeadBodyManagerImpl.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            if (con != null) {
                try {
                    con.close();
                } catch (SQLException ex) {
                    Logger.getLogger(DeadBodyManagerImpl.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
            return returnList;
        }

    }
}
